<?php


/**
 * @file
 * countdowntimer.admin.inc -contains all admin pages, settings, and validate.
 */


/**
 * Settings form for menu callback
 */
function countdowntimer_admin_settings() {

  $form = array();

  $form['countdowntimer_js_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Javascript'),
    '#collapsible' => TRUE,
  );
  $form['countdowntimer_js_settings']['countdowntimer_js_load_option'] = array(
    '#type' => 'radios',
    '#title' => t('Javascript load options'),
    '#default_value' => variable_get('countdowntimer_js_load_option', DEFAULT_JS_LOAD),
    '#options' => array(0 => t('Nodes only'), 1 => t('Every page'), 2 => t('Page list')),
    '#description' => t("Node view will only load when a single node is viewed.  Every page, is well, EVERY PAGE.  Page list will use the list below."),
  );
  $form['countdowntimer_js_settings']['countdowntimer_js_load_pages'] = array(
      '#type' => 'textarea',
      '#title' => t('Page list'),
      '#default_value' => variable_get('countdowntimer_js_load_pages', ''),
      '#description' => t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", array('%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>')),
  );


  $form['countdowntimer_output_overrides'] = array(
    '#type' => 'fieldset',
    '#title' => t('Output Overrides'),
    '#collapsible' => TRUE,
    '#description' => t("Output formats change the display of the timer.  The global format is used unless you choose a preset in the timer tags.  You can hard-code the output format in the timer tags as well.  Available replacement values are: %day%, %month%, %year%, %dow%, %moy%, %years%, %ydays%, %days%, %hours%, %mins%, and %secs%."),
  );

  $form['countdowntimer_output_overrides']['countdowntimer_output_format'] = array(
    '#type' => 'textarea',
    '#rows' => 2,
    '#title' => t('Global output format'),
    '#default_value' => variable_get('countdowntimer_output_format', DEFAULT_OUTPUT_FORMAT),
    '#description' => t("The global output format is the baseline unless you use the presets below."),
  );

  $form['countdowntimer_output_overrides']['overrides'][] = array(
  'countdowntimer_output_format_1' => array(
    '#type' => 'textarea',
    '#rows' => 2,
    '#title' => t('Output format preset 1'),
    '#default_value' => variable_get('countdowntimer_output_format_1', DEFAULT_OUTPUT_FORMAT_1),
  ),
  'countdowntimer_output_format_2' => array(
    '#type' => 'textarea',
    '#rows' => 2,
    '#title' => t('Output format preset 2'),
    '#default_value' => variable_get('countdowntimer_output_format_2', DEFAULT_OUTPUT_FORMAT_2),
  ),
  'countdowntimer_output_format_3' => array(
    '#type' => 'textarea',
    '#rows' => 2,
    '#title' => t('Output format preset 3'),
    '#default_value' => variable_get('countdowntimer_output_format_3', DEFAULT_OUTPUT_FORMAT_3),
  ),
  );

  $form['highlights'] = array(
    '#type' => 'fieldset',
    '#title' => t('Highlight settings'),
    '#description' => t('When a timer is nearing completion, you can change its\' color or use css formatting.'),
    '#tree' => FALSE,
  );
  $form['highlights'][] = array(
  'countdowntimer_highlight' => array(
    '#type' => 'textfield',
    '#size' => 100,
    '#title' => t('Highlight tagging'),
    '#default_value' => variable_get('countdowntimer_highlight', DEFAULT_HIGHLIGHT),
    '#description' => t("Use style=\"color:red;background-color:white;\" or class=\"\" etc..."),
  ),
  'countdowntimer_highlight_threshold' => array(
    '#type' => 'textfield',
    '#size' => 100,
    '#title' => t('Highlight threshold (minutes)'),
    '#default_value' => variable_get('countdowntimer_highlight_threshold', DEFAULT_HIGHLIGHT_THRESHOLD),
    '#description' => t("Number of minutes left before highlighting is applied."),
  )
  );

  $form['countdowntimer_complete_statement'] = array(
    '#type' => 'textfield',
    '#size' => 100,
    '#title' => t('Timer complete statement'),
    '#default_value' => variable_get('countdowntimer_complete_statement', DEFAULT_TIMER_COMPLETE),
    '#description' => t("Statement that prints when a timer has completed."),
  );

  $form = system_settings_form($form);
  $form['#submit'][] = 'countdowntimer_admin_settings_submit';  // add submit handler
  return $form;
}

function countdowntimer_admin_settings_submit($form, &$form_state) {
  // build timer.js file with new settings.
  countdowntimer_build_js_cache();
}

function countdowntimer_admin_settings_validate($form, &$form_state) {
  // Form submitted with single quotes
  foreach ($form_state['values'] as $key => $value) {
    if ( substr($key, 0, 15) == "countdowntimer_" ) {
      if ( ! strpos($value, "'") === FALSE ) {
        form_set_error($key, t("Countdown Timer admin settings may not contain single quotes(')."));
        drupal_goto('admin/settings/countdowntimer');
      }
    }
  }
}
