<?php

/**
 * @file
 *   Default widget implementation for an up and down timer.
 *   This widget does not use hook_ctwidget as it is always included.
 */

define('DEFAULT_OUTPUT_FORMAT', '<em>(%dow% %moy%%day%)</em><br/>%days% days + %hours%:%mins%:%secs%');
define('DEFAULT_OUTPUT_FORMAT_1', 'Only %days% days, %hours% hours, %mins% minutes and %secs% seconds left');
define('DEFAULT_OUTPUT_FORMAT_2', '%days% shopping days left');
define('DEFAULT_OUTPUT_FORMAT_3', '<em>(%dow% %moy%%day%)</em><br/>%days% days + %hours%:%mins%:%secs%');
define('DEFAULT_TIMER_COMPLETE', '<em>Timer Completed</em>');
define('DEFAULT_HIGHLIGHT', 'style="color:red"');
define('DEFAULT_HIGHLIGHT_THRESHOLD', '5');

/**
 * THIS IS NOT AN Implementation of hook_ctwidget() although it complies with the
 * hook's api.
 *
 * @returns stdClass
 *   Keys include: ->name, ->theme_function, ->js_name, and ->js_code.
 */
function ct_timer_widget() {

$ret = new stdClass();
$ret->name = 'ct_timer';
$ret->theme_function = 'ct_timer_show';
$ret->js_name = 'Drupal.countdowntimer.ct_timer';


// clean php variables for javascript injection
$output_format = "'".   countdowntimer_clean_for_javascript(variable_get('countdowntimer_output_format', DEFAULT_OUTPUT_FORMAT)) ."'";
$output_format_1 = "'". countdowntimer_clean_for_javascript(variable_get('countdowntimer_output_format_1', DEFAULT_OUTPUT_FORMAT_1)) ."'";
$output_format_2 = "'". countdowntimer_clean_for_javascript(variable_get('countdowntimer_output_format_2', DEFAULT_OUTPUT_FORMAT_2)) ."'";
$output_format_3 = "'". countdowntimer_clean_for_javascript(variable_get('countdowntimer_output_format_3', DEFAULT_OUTPUT_FORMAT_3)) ."'";
$timer_complete = "'".  countdowntimer_clean_for_javascript(variable_get('countdowntimer_complete_statement', DEFAULT_TIMER_COMPLETE)) ."'";
$highlight = "'".       countdowntimer_clean_for_javascript(variable_get('countdowntimer_highlight', DEFAULT_HIGHLIGHT)) ."'";
$threshold = "'".       countdowntimer_clean_for_javascript(variable_get('countdowntimer_highlight_threshold', DEFAULT_HIGHLIGHT_THRESHOLD)) ."'";

$ret->js_code = <<<JAVASCRIPT_CODE

/*
 * Timer widget
 */
Drupal.countdowntimer.formats = [$output_format,$output_format_1, $output_format_2, $output_format_3];

Drupal.countdowntimer.ct_timer = function() {
  this.selector = ".countdowntimer";
  this.attach = function() {
    $(this.selector).each(
      function(i) {  // i is the position in the each fieldset
        var t = new Drupal.countdowntimer.ct_timer_item(this);
        if ( t.parse_microformat_success == 1 ) {
          Drupal.countdowntimer.timer_stack[Drupal.countdowntimer.timer_stack.length] = t;
        }
      }
    );
  }
}

Drupal.countdowntimer.ct_timer_item = function(ele) {

  // defaults
  var d = {dir:"down",format_num:0, format_txt:"", timer_complete:new String($timer_complete), highlight:new String($highlight).split(/=/), threshold:new Number($threshold)};

  // .properties
  this.element = ele;
  this.d = d;

  // .methods()

  this.parse_microformat = function() {
    var timer_span = $(this.element);
    if ( timer_span.hasClass("countdowntimer") ) {
      timer_span.removeClass("countdowntimer")
    }

		// If there is an interval, always use it.
    var interval = timer_span.children("span[class=cdt_interval]");
    if ( interval.length > 0 ) {
      var interval_val = parseInt(interval.html());
      var date = new Date();
      this.to_date = date;
      this.to_date.setTime(date.getTime() + (interval_val*1000));
    } else {
      var strdate = timer_span.children("span[class=datetime]").html();
      var str_current_server_time = timer_span.children("span[class=current_server_time]").html();
      if ( String(strdate) == 'null' ) {
        this.parse_microformat_success = 0;
        throw new Object({name:"NoDate",message:"CountdownTimer: Span with class=datetime not found within the timer span."});
      }
      var date = new Date();
      try {
        date.countdowntimer_set_iso8601_date(strdate);
      }
      catch(e) {
        throw(e);
      }
      this.to_date = date;
      if ( String(str_current_server_time) != 'null' ) {
        // this is a feedback time from the server to correct for small server-client time differences.
        // not used for normal block and node timers.
        var date_server = new Date();
        date_server.countdowntimer_set_iso8601_date(str_current_server_time);
        var date_client = new Date();
        var adj = date_client.getTime() - date_server.getTime();
        // adjust target date to clients domain
        this.to_date.setTime(this.to_date.getTime() + adj);
      }
    }

    // common attributes
    this.d.dir = timer_span.children("span[class=dir]").html() || this.d.dir;
    this.d.format_num = timer_span.children("span[class=format_num]").html() || this.d.format_num;
    this.d.format_txt = timer_span.children("span[class=format_txt]").html() || "";
    if ( String(this.d.format_txt).match("'") ) {
      this.d.format_txt = "<span style=\"color:red;\">Format may not contain single quotes(').</span>";
    }
    this.d.threshold = timer_span.children("span[class=threshold]").html() || this.d.threshold;
    this.d.timer_complete = timer_span.children("span[class=complete]").html() || this.d.timer_complete;
    this.d.tc_redir = timer_span.children("span[class=tc_redir]").html() || '';
    this.d.tc_msg = timer_span.children("span[class=tc_msg]").html() || '';

    this.parse_microformat_success = 1;
  }



  this.update = function() {
    var timer_span = $(this.element);
    var now_date = new Date();
    var diff_secs;
    if ( this.d.dir == "down" ) {
      diff_secs = Math.floor((this.to_date.getTime() - now_date.getTime()) / 1000);
    } else {
      diff_secs = Math.floor((now_date.getTime() - this.to_date.getTime()) / 1000);
    }

    if ( this.d.dir == "down" && diff_secs < 0 ) {
      /* timer complete */
      timer_span.html(this.d.timer_complete.valueOf());

      if ( this.d.tc_msg != '' && diff_secs > -3 ) {
        alert(this.d.tc_msg);
        if ( this.d.tc_redir != '' ) {
          window.location = this.d.tc_redir;
        }
      } else if ( this.d.tc_redir != '' && diff_secs > -3) {
        window.location = this.d.tc_redir;
      }

      return false;
    } else {
      /* timer still counting */
      var years = Math.floor(diff_secs / 60 / 60 / 24 / 365.25);
      var days = Math.floor(diff_secs / 60 / 60 / 24);
      var ydays = Math.ceil(days - (years * 365.25));
      var hours = Math.floor(diff_secs / 60 / 60) - (days * 24);
      var minutes = Math.floor(diff_secs / 60) - (hours * 60) - (days * 24 * 60);
      var seconds = diff_secs - (minutes * 60) - (hours * 60 * 60) - (days * 24 * 60 * 60);

      var outhtml = new String(this.outformat);

      // try to handle counts with units first, use a try block because Drupal.formatPlural breaks javascript sometimes
      try {
        outhtml = outhtml.replace(/%years% years/,  Drupal.formatPlural(years, "1 year", "@count years"));
        outhtml = outhtml.replace(/%ydays% days/,   Drupal.formatPlural(ydays, "1 day", "@count days"));
        outhtml = outhtml.replace(/%days% days/,    Drupal.formatPlural(days, "1 day", "@count days"));
        outhtml = outhtml.replace(/%hours% hours/,  Drupal.formatPlural(hours, "1 hour", "@count hours"));
        outhtml = outhtml.replace(/%mins% minutes/, Drupal.formatPlural(minutes, "1 minute", "@count minutes"));
        outhtml = outhtml.replace(/%secs% seconds/, Drupal.formatPlural(seconds, "1 second", "@count seconds"));
      }
      catch(e){
        // suppress errors
      }

      //handle counts without units
      outhtml = outhtml.replace(/%years%/, years);
      outhtml = outhtml.replace(/%ydays%/, ydays);
      outhtml = outhtml.replace(/%days%/, days);
      outhtml = outhtml.replace(/%hours%/, LZ(hours));
      outhtml = outhtml.replace(/%mins%/, LZ(minutes));
      outhtml = outhtml.replace(/%secs%/, LZ(seconds));
      outhtml = outhtml.replace(/%hours_nopad%/, hours);
      outhtml = outhtml.replace(/%mins_nopad%/, minutes);
      outhtml = outhtml.replace(/%secs_nopad%/, seconds);

      if ( this.d.dir == "down" && (diff_secs <= (this.d.threshold * 60)) ) {
        timer_span.html('<span ' + this.d.highlight[0] + '=' + this.d.highlight[1] + '>' +  outhtml + '</span>');
      } else {
        timer_span.html(outhtml);
      }

      return true;
    }
  }


  /* bootstrap, parse microformat, load object */
  try {
    this.parse_microformat();
  }
  catch(e) {
    alert(e.message);
    alert($(ele).html());
    this.parse_microformat_success = 0;
    return;
  }

  if ( d.format_txt != "" ) {
      this.outformat = d.format_txt;
  } else {
    this.outformat = Drupal.countdowntimer.formats[d.format_num];
  }

  // replace the static stuff in the format string
  // this only needs to be done once, so here is a good spot.
  this.outformat = this.outformat.replace(/%day%/,   this.to_date.getDate());
  this.outformat = this.outformat.replace(/%month%/, this.to_date.getMonth() + 1);
  this.outformat = this.outformat.replace(/%year%/,  this.to_date.getFullYear());
  this.outformat = this.outformat.replace(/%moy%/,   this.to_date.countdowntimer_get_moy());
  this.outformat = this.outformat.replace(/%dow%/,   this.to_date.countdowntimer_get_dow());
}
// End of timer widget.

JAVASCRIPT_CODE;

return $ret;
}


function ct_timer_show($widget_args) {
  $output = '<span class="countdowntimer">';
  $valid_atts = array('datetime', 'dir', 'format_txt', 'format_num', 'threshold', 'complete', 'tc_redir', 'tc_msg', 'cdt_interval', 'current_server_time');
  foreach ($valid_atts as $att) {
    if ( isset($widget_args[$att]) ) {
      $output .= '<span style="display:none" class="'. $att. '">'. $widget_args[$att] .'</span>';
    }
  }
  $output .= '</span>';
  return $output;
}